/*
 * Created on May 10, 2005 Used to retrieve stored file paths for seismograms in
 * database
 */
package edu.sc.seis.cormorant.seismogram;

import org.apache.log4j.Logger;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.UNKNOWN;

import edu.iris.Fissures.FissuresException;
import edu.iris.Fissures.Time;
import edu.iris.Fissures.IfSeismogramDC.DataCenterCallBack;
import edu.iris.Fissures.IfSeismogramDC.DataCenterPOA;
import edu.iris.Fissures.IfSeismogramDC.LocalSeismogram;
import edu.iris.Fissures.IfSeismogramDC.RequestFilter;
import edu.iris.Fissures.model.MicroSecondDate;
import edu.iris.Fissures.model.UnitImpl;
import edu.iris.Fissures.network.ChannelIdUtil;
import edu.sc.seis.fissuresUtil.hibernate.SeismogramFileRefDB;
import edu.sc.seis.fissuresUtil.time.RangeTool;

/**
 * @author fenner Used to retrieve stored file paths for siesmograms in database
 * 
 */
public class SodDBSeismogramServer extends DataCenterPOA {

    public SodDBSeismogramServer(boolean ignoreNetworkTimes) {
        this.seisFileDb = SeismogramFileRefDB.getSingleton();
        this.ignoreNetworkTimes = ignoreNetworkTimes;
    }

    public RequestFilter[] available_data(RequestFilter[] requestArray) {
        try {
            checkForBadRequests(requestArray);
            logger.debug("Request covering " + RangeTool.getFullTime(requestArray));
            synchronized(seisFileDb) {
                RequestFilter[] requestResults = seisFileDb.findMatchingSeismograms(requestArray,
                                                                                      ignoreNetworkTimes);
                logger.debug("Result array length:  " + requestResults.length);
                return requestResults;
            }
        } catch(Throwable e) {
            logger.error("Exception occured while finding matching seismograms in database using findMatchingSeismograms",
                         e);
            throw new UNKNOWN(e.toString());
        }
    }

    public LocalSeismogram[] retrieve_seismograms(RequestFilter[] requestArray)
            throws FissuresException {
        checkForBadRequests(requestArray);
        try {
            logger.debug("Request array length: " + requestArray.length);
            synchronized(seisFileDb) {
                LocalSeismogram[] requestResults = seisFileDb.getMatchingSeismograms(requestArray,
                                                                                       ignoreNetworkTimes);
                logger.debug("Result array length:  " + requestResults.length);
                return requestResults;
            }
        } catch(Throwable e) {
            logger.error("Exception occured while getting matching seismograms in database using getMatchingSeismograms",
                         e);
            throw new UNKNOWN(e.toString());
        }
    }

    public static void checkForBadRequests(RequestFilter[] a_filterseq)
            throws FissuresException {
        for(int i = 0; i < a_filterseq.length; i++) {
            MicroSecondDate first = new MicroSecondDate(a_filterseq[i].start_time);
            MicroSecondDate last = new MicroSecondDate(a_filterseq[i].end_time);
            if(first.after(last)) {
                // probably misconfigured request
                String msg = "available_data first after last "
                        + ChannelIdUtil.toString(a_filterseq[i].channel_id)
                        + "   from " + a_filterseq[i].start_time.date_time
                        + "   to   " + a_filterseq[i].end_time.date_time;
                logger.info(msg);
                throw new FissuresException(msg,
                                            new edu.iris.Fissures.Error(5, msg));
            } // end of if (first.after(last))
            if(last.difference(first).convertTo(UnitImpl.HOUR).value > 25) {
                String msg = "Data request too large: "
                        + ChannelIdUtil.toString(a_filterseq[i].channel_id)
                        + a_filterseq[i].start_time.date_time + "   to   "
                        + a_filterseq[i].end_time.date_time;
                logger.warn(msg);
                throw new org.omg.CORBA.IMP_LIMIT(msg,
                                                  0,
                                                  org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        }
    }

    public String request_seismograms(RequestFilter[] arg0,
                                      DataCenterCallBack arg1,
                                      boolean arg2,
                                      Time arg3) {
        throw new NO_IMPLEMENT();
    }

    public String queue_seismograms(RequestFilter[] arg0)
            throws FissuresException {
        throw new NO_IMPLEMENT();
    }

    public LocalSeismogram[] retrieve_queue(String arg0)
            throws FissuresException {
        throw new NO_IMPLEMENT();
    }

    public void cancel_request(String arg0) {
        throw new NO_IMPLEMENT();
    }

    public String request_status(String arg0) {
        throw new NO_IMPLEMENT();
    }

    private static final Logger logger = Logger.getLogger(SodDBSeismogramServer.class);

    private SeismogramFileRefDB seisFileDb;

    private boolean ignoreNetworkTimes;
}
